@@ -29,6 +29,8 @@
 
 #include <algorithm>
 
+using namespace std::literals;
+
 class CTreeItemData : public wxTreeItemData
 {
 public:
@@ -60,7 +62,7 @@ public:
 
 	wxTreeItemId GetHit(const wxPoint& point)
 	{
-		int flags = 0;
+		int flags{};
 		wxTreeItemId hit = m_pLocalTreeView->HitTest(point, flags);
 
 		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
@@ -242,7 +244,7 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, C
 	, COptionChangeEventHandler(this)
 	, m_pQueueView(pQueueView)
 {
-	wxGetApp().AddStartupProfileRecord("CLocalTreeView::CLocalTreeView");
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::CLocalTreeView"sv);
 #ifdef __WXMAC__
 	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
 #endif
@@ -414,7 +416,7 @@ wxTreeItemId CLocalTreeView::GetSubdir(wxTreeItemId parent, wxString const& subD
 
 bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 {
-	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives");
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives"sv);
 
 	std::vector<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 
@@ -436,7 +438,7 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 	}
 	SortChildren(parent);
 
-	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives adding drives done");
+	wxGetApp().AddStartupProfileRecord("CLocalTreeView::DisplayDrives adding drives done"sv);
 
 	return true;
 }
@@ -912,7 +914,7 @@ void CLocalTreeView::OnStateChange(t_statechange_notifications notification, std
 		SetDir(m_state.GetLocalDir().GetPath());
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
@@ -1103,10 +1105,10 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 
 	wxMenu menu;
 	auto item = new wxMenuItem(&menu, XRCID("ID_UPLOAD"), _("&Upload"), _("Upload selected directory"));
-	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU));
+	item->SetBitmap(MakeBmpBundle(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU)));
 	menu.Append(item);
 	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
-	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU));
+	item->SetBitmap(MakeBmpBundle(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU)));
 	menu.Append(item);
 
 	menu.AppendSeparator();
