@@ -89,7 +89,7 @@ public:
 		auto const format = GetReceivedFormat();
 		if (format == m_pFileDataObject->GetFormat() || format == LocalDataObjectFormat()) {
 			std::wstring subdir;
-			int flags = 0;
+			int flags{};
 			int hit = m_pRemoteListView->HitTest(wxPoint(x, y), flags, 0);
 			if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 				int index = m_pRemoteListView->GetItemIndex(hit);
@@ -126,7 +126,7 @@ public:
 
 		// Find drop directory (if it exists)
 		std::wstring subdir;
-		int flags = 0;
+		int flags{};
 		int hit = m_pRemoteListView->HitTest(wxPoint(x, y), flags, 0);
 		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 			int index = m_pRemoteListView->GetItemIndex(hit);
@@ -204,7 +204,7 @@ public:
 
 	int DoDisplayDropHighlight(wxPoint point)
 	{
-		int flags;
+		int flags{};
 		int hit = m_pRemoteListView->HitTest(point, flags, 0);
 		if (!(flags & wxLIST_HITTEST_ONITEM)) {
 			hit = -1;
@@ -369,21 +369,18 @@ CRemoteListView::~CRemoteListView()
 int CRemoteListView::OnGetItemImage(long item) const
 {
 	CRemoteListView *pThis = const_cast<CRemoteListView *>(this);
+
 	int index = GetItemIndex(item);
-	if (index < 0) {
+	if (index < 0 || static_cast<size_t>(index) >= pThis->m_fileData.size()) {
 		return -1;
 	}
 
-	if (index >= pThis->m_fileData.size()) {
-		return -1;
-	}
 	int &icon = pThis->m_fileData[index].icon;
-
 	if (icon != -2) {
 		return icon;
 	}
 
-	if (!m_pDirectoryListing || index >= m_pDirectoryListing->size()) {
+	if (!m_pDirectoryListing || static_cast<size_t>(index) >= m_pDirectoryListing->size()) {
 		return -1;
 	}
 
@@ -1050,10 +1047,10 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 
 	wxMenu menu;
 	auto item = new wxMenuItem(&menu, XRCID("ID_DOWNLOAD"), _("&Download"), _("Download selected files and directories"));
-	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOAD"), wxART_MENU));
+	item->SetBitmap(MakeBmpBundle(wxArtProvider::GetBitmap(_T("ART_DOWNLOAD"), wxART_MENU)));
 	menu.Append(item);
 	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add files to queue"), _("Add selected files and folders to the transfer queue"));
-	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOADADD"), wxART_MENU));
+	item->SetBitmap(MakeBmpBundle(wxArtProvider::GetBitmap(_T("ART_DOWNLOADADD"), wxART_MENU)));
 	menu.Append(item);
 	menu.Append(XRCID("ID_ENTER"), _("E&nter directory"), _("Enter selected directory"));
 	menu.Append(XRCID("ID_EDIT"), _("&View/Edit"));
@@ -2111,10 +2108,10 @@ void CRemoteListView::OnStateChange(t_statechange_notifications notification, st
 	}
 	else if (notification == STATECHANGE_SERVER) {
 		if (m_windowTinter) {
-			m_windowTinter->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
+			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
 		}
 		if (m_pInfoText) {
-			m_pInfoText->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
+			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
 		}
 	}
 	else {
@@ -2623,6 +2620,9 @@ void CRemoteListView::OnExitComparisonMode()
 
 bool CRemoteListView::ItemIsDir(int index) const
 {
+	if (!m_pDirectoryListing || index < 0 || index >= m_pDirectoryListing->size()) {
+		return false;
+	}
 	return (*m_pDirectoryListing)[index].is_dir();
 }
 
@@ -2813,6 +2813,6 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		return;
 	}
 
-	CFileTransferCommand *cmd = new CFileTransferCommand(memory_reader_factory(L"Empty file", std::string_view()), m_pDirectoryListing->path, newFileName, transfer_flags());
+	CFileTransferCommand *cmd = new CFileTransferCommand(fz::view_reader_factory(L"Empty file", std::string_view()), m_pDirectoryListing->path, newFileName, transfer_flags());
 	m_state.m_pCommandQueue->ProcessCommand(cmd);
 }
