@@ -39,6 +39,7 @@ EVT_LEFT_DOWN(CFileListCtrl<CFileData>::OnLeftDown)
 EVT_COMMAND(wxID_ANY, fz_EVT_DEFERRED_MOUSEEVENT, CFileListCtrl<CFileData>::OnProcessMouseEvent)
 #endif
 EVT_KEY_DOWN(CFileListCtrl<CFileData>::OnKeyDown)
+EVT_SYS_COLOUR_CHANGED(CFileListCtrl<CFileData>::OnColorChange)
 END_EVENT_TABLE()
 
 #ifdef __WXMSW__
@@ -180,7 +181,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 		LV_ITEM& lvi = info->item;
 		long item = lvi.iItem;
 
-		int column = m_pVisibleColumnMapping[lvi.iSubItem];
+		int column = GetColumnActualIndex(lvi.iSubItem);
 
 		if (lvi.mask & LVIF_TEXT) {
 			wxString text = GetItemText(item, column);
@@ -250,10 +251,11 @@ static gboolean gtk_button_release_event(GtkWidget*, void *gdk_event, CGtkEventC
 }
 #endif
 
-template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CQueueView* pQueue, bool border)
+template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CQueueView* pQueue, COptionsBase & options, bool border)
 	: wxListCtrlEx(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxLC_VIRTUAL | wxLC_REPORT | wxLC_EDIT_LABELS | (border ? wxBORDER_SUNKEN : wxNO_BORDER))
 	, CComparableListing(this)
 	, m_pQueue(pQueue)
+	, options_(options)
 {
 	CreateSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x);
 
@@ -281,6 +283,16 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 #ifndef __WXMSW__
 	GetMainWindow()->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 #endif
+
+#ifdef __WXMSW__
+	Bind(wxEVT_SYS_COLOUR_CHANGED, [this](wxSysColourChangedEvent& evt) {
+		// Delay refresh until parent had time to process change.
+		CallAfter([this](){
+			InitColors();
+		});
+		evt.Skip();
+	});
+#endif
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=-1*/, int direction /*=-1*/, bool updateSelections /*=true*/)
@@ -346,7 +358,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		}
 	}
 
-	const int dirSortOption = COptions::Get()->get_int(OPTION_FILELIST_DIRSORT);
+	const int dirSortOption = options_.get_int(OPTION_FILELIST_DIRSORT);
 
 	if (column == m_sortColumn && direction != m_sortDirection && !m_indexMapping.empty() &&
 		dirSortOption != 1)
@@ -381,7 +393,8 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	if (m_hasParent) {
 		++start;
 	}
-	std::unique_ptr<CFileListCtrlSortBase> object = GetSortComparisonObject();
+	UpdateSortComparisonObject();
+	auto & object = GetSortComparisonObject();
 	std::sort(start, m_indexMapping.end(), SortPredicate(object));
 
 	if (updateSelections) {
@@ -419,7 +432,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelectio
 
 template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFileData>::GetDirSortMode()
 {
-	const int dirSortOption = COptions::Get()->get_int(OPTION_FILELIST_DIRSORT);
+	const int dirSortOption = options_.get_int(OPTION_FILELIST_DIRSORT);
 
 	CFileListCtrlSortBase::DirSortMode dirSortMode;
 	switch (dirSortOption)
@@ -446,7 +459,7 @@ template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFile
 
 template<class CFileData> NameSortMode CFileListCtrl<CFileData>::GetNameSortMode()
 {
-	const int nameSortOption = COptions::Get()->get_int(OPTION_FILELIST_NAMESORT);
+	const int nameSortOption = options_.get_int(OPTION_FILELIST_NAMESORT);
 
 	NameSortMode nameSortMode;
 	switch (nameSortOption)
@@ -468,7 +481,7 @@ template<class CFileData> NameSortMode CFileListCtrl<CFileData>::GetNameSortMode
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListEvent &event)
 {
-	int col = m_pVisibleColumnMapping[event.GetColumn()];
+	int col = GetColumnActualIndex(event.GetColumn());
 	if (col == -1) {
 		return;
 	}
@@ -502,13 +515,13 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListE
 
 #ifdef __WXMSW__
 namespace {
-wxString GetExt(const wxString& file)
+std::wstring GetExt(std::wstring const& file)
 {
-	wxString ret;
+	std::wstring ret;
 
-	int pos = file.Find('.', true);
-	if (pos > 0 && (static_cast<size_t>(pos) + 1) < file.size()) { // Does neither starts nor end with dot
-		ret = file.Mid(pos + 1);
+	size_t pos = file.rfind('.');
+	if (pos != std::wstring::npos && pos > 0 && pos + 1 < file.size()) { // Does neither starts nor end with dot
+		ret = file.substr(pos + 1);
 	}
 
 	return ret;
@@ -516,10 +529,11 @@ wxString GetExt(const wxString& file)
 }
 #endif
 
-template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::wstring name, bool dir, std::wstring const& path)
+template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::wstring const& name, bool dir, std::wstring const& path)
 {
+	static std::wstring const generic_suffix = L"-" + _("file").ToStdWstring();
 #ifdef __WXMSW__
-	wxString ext;
+	std::wstring ext;
 	if (dir) {
 		if (!path.empty()) {
 			ext = '/';
@@ -527,28 +541,34 @@ template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::ws
 	}
 	else {
 		ext = GetExt(name);
-		ext.MakeLower();
+		fz::str_tolower_inplace(ext);
 	}
 	auto typeIter = m_fileTypeMap.find(ext);
 	if (typeIter != m_fileTypeMap.end()) {
 		return typeIter->second;
 	}
 
-	wxString type;
+	std::wstring type;
+
+	std::wstring fullname;
 	int flags = SHGFI_TYPENAME;
 	if (path.empty()) {
 		flags |= SHGFI_USEFILEATTRIBUTES;
 	}
 	else if (path == _T("\\")) {
-		name += _T("\\");
+		fullname.reserve(name.size() + 1);
+		fullname = name;
+		fullname += '\\';
 	}
 	else {
-		name = path + name;
+		fullname.reserve(name.size() + path.size());
+		fullname = path;
+		fullname += name;
 	}
 
 	SHFILEINFO shFinfo;
 	memset(&shFinfo, 0, sizeof(SHFILEINFO));
-	if (SHGetFileInfo(name.c_str(),
+	if (SHGetFileInfo(path.empty() ? name.c_str() : fullname.c_str(),
 		dir ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
 		&shFinfo,
 		sizeof(shFinfo),
@@ -556,10 +576,10 @@ template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::ws
 	{
 		if (!*shFinfo.szTypeName) {
 			if (!ext.empty()) {
+				type.reserve(ext.size() + generic_suffix.size());
 				type = ext;
-				type.MakeUpper();
-				type += _T("-");
-				type += _("file");
+				fz::str_toupper_inplace(ext);
+				type += generic_suffix;
 			}
 			else {
 				type = m_genericTypes[genericTypes::file];
@@ -568,22 +588,22 @@ template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::ws
 		else {
 			type = shFinfo.szTypeName;
 			if (!ext.empty()) {
-				m_fileTypeMap[ext.MakeLower()] = type.ToStdWstring();
+				m_fileTypeMap[ext] = type;
 			}
 		}
 	}
 	else {
 		if (!ext.empty()) {
+			type.reserve(ext.size() + generic_suffix.size());
 			type = ext;
-			type.MakeUpper();
-			type += _T("-");
-			type += _("file");
+			fz::str_toupper_inplace(ext);
+			type += generic_suffix;
 		}
 		else {
 			type = m_genericTypes[genericTypes::file];
 		}
 	}
-	return type.ToStdWstring();
+	return type;
 #else
 	(void)path;
 
@@ -595,36 +615,38 @@ template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::ws
 	if (pos == std::wstring::npos || pos < 1 || !name[pos + 1]) { // No dot or starts or ends with dot
 		return m_genericTypes[genericTypes::file];
 	}
-	wxString ext = name.substr(pos + 1);
-	wxString lower_ext = ext.Lower();
+	std::wstring ext = name.substr(pos + 1);
+	std::wstring lower_ext = fz::str_tolower(ext);
 
 	auto typeIter = m_fileTypeMap.find(lower_ext);
 	if (typeIter != m_fileTypeMap.end()) {
 		return typeIter->second;
 	}
 
+	std::wstring type;
+
 	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
 	if (!pType) {
-		wxString desc = ext;
-		desc += _T("-");
-		desc += _("file");
-		m_fileTypeMap[ext] = desc.ToStdWstring();
-		return desc.ToStdWstring();
+		type.reserve(ext.size() + generic_suffix.size());
+		type = ext;
+		type += generic_suffix;
+		m_fileTypeMap[ext] = type;
+		return type;
 	}
 
 	wxString desc;
 	if (pType->GetDescription(&desc) && !desc.empty()) {
 		delete pType;
-		m_fileTypeMap[ext] = desc.ToStdWstring();
-		return desc.ToStdWstring();
+		type = desc.ToStdWstring();
+		m_fileTypeMap[lower_ext] = type;
+		return type;
 	}
 	delete pType;
-
-	desc = ext;
-	desc += _T("-");
-	desc += _("file");
-	m_fileTypeMap[lower_ext] = desc.ToStdWstring();
-	return desc.ToStdWstring();
+	type.reserve(ext.size() + generic_suffix.size());
+	type = ext;
+	type += generic_suffix;
+	m_fileTypeMap[lower_ext] = type;
+	return type;
 #endif
 }
 
@@ -769,7 +791,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wx
 
 template<class CFileData> void CFileListCtrl<CFileData>::InitSort(interfaceOptions optionID)
 {
-	wxString sortInfo = COptions::Get()->get_string(optionID);
+	wxString sortInfo = options_.get_string(optionID);
 	if (!sortInfo.empty()) {
 		m_sortDirection = sortInfo[0] - '0';
 	}
@@ -855,6 +877,59 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnItemDeselected(wxList
 #endif
 
 	const int item = event.GetIndex();
+
+#if !defined(__WXMSW__) && wxCHECK_VERSION(3, 2, 1)
+	if (item == -1) {
+		if (wxGetKeyState(WXK_SHIFT)) {
+			// Sad, so expensive.
+			if (m_pending_focus_processing) {
+				updateAllSelections_ = true;
+			}
+			else {
+				UpdateSelections(0, static_cast<int>(m_selections.size()) - 1);
+			}
+		}
+		else {
+			// Everything deselected...
+			for (unsigned int i = 0; i < m_selections.size(); ++i) {
+				m_selections[i] = false;
+			}
+			if (m_pFilelistStatusBar) {
+				m_pFilelistStatusBar->UnselectAll();
+			}
+
+			// ...but possibly the focus
+			if (m_hasParent && !m_focusItem) {
+				return;
+			}
+			if (m_focusItem >= 0 && m_focusItem < GetItemCount()) {
+				bool selected = GetItemState(m_focusItem, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
+				if (selected) {
+					m_selections[m_focusItem] = true;
+					if (m_focusItem >= m_indexMapping.size()) {
+						return;
+					}
+					const int index = m_indexMapping[m_focusItem];
+					const CFileData& data = m_fileData[index];
+					if (data.comparison_flags == fill) {
+						return;
+					}
+
+					if (m_pFilelistStatusBar) {
+						if (ItemIsDir(index)) {
+							m_pFilelistStatusBar->SelectDirectory();
+						}
+						else {
+							m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
+						}
+					}
+				}
+			}
+		}
+		return;
+	}
+#endif
+
 	if (item < 0 || item >= (int)m_indexMapping.size()) {
 		return;
 	}
@@ -926,6 +1001,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::SetItemCount(int count)
 template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wxCommandEvent& event)
 {
 	m_pending_focus_processing--;
+	if (updateAllSelections_) {
+		updateAllSelections_ = false;
+		UpdateSelections(0, static_cast<int>(m_selections.size()) - 1);
+		return;
+	}
+
 	int old_focus = event.GetInt();
 	int new_focus = (int)event.GetExtraLong();
 
@@ -1046,13 +1127,18 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxC
 	}
 
 	bool selected = GetItemState(m_focusItem, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
-	if (!selected && m_selections[m_focusItem]) {
-		// Need to deselect all
-		if (m_pFilelistStatusBar) {
-			m_pFilelistStatusBar->UnselectAll();
+	if (!selected) {
+		if (wxGetKeyState(WXK_CONTROL)) {
+			UpdateSelections(0, static_cast<int>(m_selections.size()) - 1);
 		}
-		for (unsigned int i = 0; i < m_selections.size(); ++i) {
-			m_selections[i] = 0;
+		else {
+			// Need to deselect all
+			if (m_pFilelistStatusBar) {
+				m_pFilelistStatusBar->UnselectAll();
+			}
+			for (unsigned int i = 0; i < m_selections.size(); ++i) {
+				m_selections[i] = 0;
+			}
 		}
 	}
 }
@@ -1146,3 +1232,20 @@ template<class CFileData> void CFileListCtrl<CFileData>::UpdateSelections_ItemsA
 		}
 	}
 }
+
+template<class CFileData> CFileListCtrlSortBase& CFileListCtrl<CFileData>::GetSortComparisonObject()
+{
+	if (!sortComparisonObject_) {
+		UpdateSortComparisonObject();
+	}
+
+	return *sortComparisonObject_;
+}
+
+template<class CFileData> void CFileListCtrl<CFileData>::OnColorChange(wxSysColourChangedEvent & ev)
+{
+	InitColors();
+	Refresh();
+	ev.Skip();
+}
+
