@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "queue_storage.h"
 #include "Options.h"
 #include "queue.h"
@@ -14,7 +14,8 @@
 enum class Column_type
 {
 	text,
-	integer
+	integer,
+	blob
 };
 
 enum _column_flags
@@ -89,12 +90,13 @@ namespace file_table_column_names
 		target_file,
 		local_path,
 		remote_path,
-		download,
 		size,
 		error_count,
 		priority,
-		ascii_file,
-		default_exists_action
+		flags,
+		default_exists_action,
+		extra_flags,
+		persistent_state
 	};
 }
 
@@ -105,12 +107,13 @@ _column file_table_columns[] = {
 	{ "target_file", Column_type::text, 0 },
 	{ "local_path", Column_type::integer, 0 },
 	{ "remote_path", Column_type::integer, 0 },
-	{ "download", Column_type::integer, not_null },
 	{ "size", Column_type::integer, 0 },
 	{ "error_count", Column_type::integer, 0 },
 	{ "priority", Column_type::integer, 0 },
-	{ "ascii_file", Column_type::integer, 0 },
-	{ "default_exists_action", Column_type::integer, 0 }
+	{ "flags", Column_type::integer, 0 },
+	{ "default_exists_action", Column_type::integer, 0 },
+	{ "extra_flags", Column_type::text, 0 },
+	{ "persistent_state", Column_type::blob, 0 }
 };
 
 namespace path_table_column_names
@@ -156,10 +159,12 @@ public:
 	bool Bind(sqlite3_stmt* statement, int index, std::wstring const& value);
 	bool Bind(sqlite3_stmt* statement, int index, std::string const& value);
 	bool Bind(sqlite3_stmt* statement, int index, const char* const value);
+	bool BindBlob(sqlite3_stmt* statement, int index, std::string const& value);
 	bool BindNull(sqlite3_stmt* statement, int index);
 
 	std::wstring GetColumnText(sqlite3_stmt* statement, int index);
 	std::string GetColumnTextUtf8(sqlite3_stmt* statement, int index);
+	std::string GetColumnBlob(sqlite3_stmt* statement, int index);
 	int64_t GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def = 0);
 	int GetColumnInt(sqlite3_stmt* statement, int index, int def = 0);
 
@@ -295,7 +300,7 @@ bool CQueueStorage::Impl::MigrateSchema()
 	bool ret = sqlite3_exec(db_, "PRAGMA user_version", int_callback, &version, 0) == SQLITE_OK;
 
 	if (ret) {
-		if (version > 5) {
+		if (version > 8) {
 			ret = false;
 		}
 		else if (version > 0) {
@@ -309,9 +314,37 @@ bool CQueueStorage::Impl::MigrateSchema()
 			if (ret && version < 5) {
 				ret = sqlite3_exec(db_, "ALTER TABLE servers ADD COLUMN site_path TEXT DEFAULT NULL", 0, 0, 0) == SQLITE_OK;
 			}
+			if (ret && version < 6) {
+				std::string query("CREATE TABLE IF NOT EXISTS files2 ");
+				query += CreateColumnDefs(file_table_columns, sizeof(file_table_columns) / sizeof(_column));
+				ret = sqlite3_exec(db_, query.c_str(), 0, 0, 0) == SQLITE_OK;
+				ret &= sqlite3_exec(db_, "CREATE INDEX IF NOT EXISTS server_index ON files2 (server)", 0, 0, 0) == SQLITE_OK;
+				query = fz::sprintf(
+					"INSERT INTO files2 (id, server, source_file, target_file, local_path, remote_path, size, error_count, priority, default_exists_action, flags) "
+					"SELECT id, server, source_file, target_file, local_path, remote_path, size, error_count, priority, default_exists_action, download * %d + ascii_file * %d FROM files"
+					, transfer_flags::download, ftp_transfer_flags::ascii);
+				ret &= sqlite3_exec(db_, query.c_str(), 0, 0, 0) == SQLITE_OK;
+				ret &= sqlite3_exec(db_, "DROP TABLE files", 0, 0, 0) == SQLITE_OK;
+				ret &= sqlite3_exec(db_, "ALTER TABLE files2 RENAME TO files", 0, 0, 0) == SQLITE_OK;
+			}
+			if (ret && version < 7) {
+				std::string query("CREATE TABLE IF NOT EXISTS files2 ");
+				query += CreateColumnDefs(file_table_columns, sizeof(file_table_columns) / sizeof(_column));
+				ret = sqlite3_exec(db_, query.c_str(), 0, 0, 0) == SQLITE_OK;
+				ret &= sqlite3_exec(db_, "CREATE INDEX IF NOT EXISTS server_index ON files2 (server)", 0, 0, 0) == SQLITE_OK;
+				query =
+					"INSERT INTO files2 (id, server, source_file, target_file, local_path, remote_path, size, error_count, priority, default_exists_action, flags) "
+					"SELECT id, server, source_file, target_file, local_path, remote_path, size, error_count, priority, default_exists_action, flags FROM files";
+				ret &= sqlite3_exec(db_, query.c_str(), 0, 0, 0) == SQLITE_OK;
+				ret &= sqlite3_exec(db_, "DROP TABLE files", 0, 0, 0) == SQLITE_OK;
+				ret &= sqlite3_exec(db_, "ALTER TABLE files2 RENAME TO files", 0, 0, 0) == SQLITE_OK;
+			}
+			if (ret && version < 8) {
+				ret = sqlite3_exec(db_, "ALTER TABLE files ADD COLUMN persistent_state BLOB DEFAULT NULL", 0, 0, 0) == SQLITE_OK;
+			}
 		}
-		if (ret && version != 5) {
-			ret = sqlite3_exec(db_, "PRAGMA user_version = 5", 0, 0, 0) == SQLITE_OK;
+		if (ret && version != 8) {
+			ret = sqlite3_exec(db_, "PRAGMA user_version = 8", 0, 0, 0) == SQLITE_OK;
 		}
 	}
 
@@ -397,6 +430,9 @@ std::string CQueueStorage::Impl::CreateColumnDefs(_column const* columns, size_t
 		if (columns[i].type == Column_type::integer) {
 			query += " INTEGER";
 		}
+		else if (columns[i].type == Column_type::blob) {
+			query += " BLOB";
+		}
 		else {
 			query += " TEXT";
 		}
@@ -611,10 +647,15 @@ bool CQueueStorage::Impl::BindNull(sqlite3_stmt* statement, int index)
 	return sqlite3_bind_null(statement, index) == SQLITE_OK;
 }
 
+bool CQueueStorage::Impl::BindBlob(sqlite3_stmt* statement, int index, std::string const& value)
+{
+	return sqlite3_bind_blob(statement, index, value.c_str(), value.size(), SQLITE_TRANSIENT) == SQLITE_OK;
+}
+
 
 bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 {
-	bool kiosk_mode = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
+	bool kiosk_mode = COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) != 0;
 
 	Site const& site = item.GetSite();
 
@@ -624,13 +665,13 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 	Bind(insertServerQuery_, server_table_column_names::type, static_cast<int>(site.server.GetType()));
 
 	ProtectedCredentials credentials = site.credentials;
-	credentials.Protect();
+	protect(credentials);
 
 	LogonType logonType = credentials.logonType_;
 	if (logonType != LogonType::anonymous) {
 		Bind(insertServerQuery_, server_table_column_names::user, site.server.GetUser());
 
-		if (logonType == LogonType::normal || logonType == LogonType::account) {
+		if (logonType == LogonType::normal || logonType == LogonType::account || logonType == LogonType::profile) {
 			if (kiosk_mode) {
 				logonType = LogonType::ask;
 				BindNull(insertServerQuery_, server_table_column_names::password);
@@ -732,8 +773,8 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 	}
 
 	Bind(insertServerQuery_, server_table_column_names::bypass_proxy, site.server.GetBypassProxy() ? 1 : 0);
-	if (!site.server.GetName().empty()) {
-		Bind(insertServerQuery_, server_table_column_names::name, site.server.GetName());
+	if (!site.GetName().empty()) {
+		Bind(insertServerQuery_, server_table_column_names::name, site.GetName());
 	}
 	else {
 		BindNull(insertServerQuery_, server_table_column_names::name);
@@ -790,12 +831,32 @@ bool CQueueStorage::Impl::SaveFile(CFileItem const& file)
 	}
 
 	Bind(insertFileQuery_, file_table_column_names::source_file, file.GetSourceFile());
-	auto const& targetFile = file.GetTargetFile();
-	if (targetFile) {
-		Bind(insertFileQuery_, file_table_column_names::target_file, *targetFile);
+	auto const& extra_data = file.GetExtraData();
+	if (extra_data) {
+		if (!extra_data->targetFile_.empty()) {
+			Bind(insertFileQuery_, file_table_column_names::target_file, extra_data->targetFile_);
+		}
+		else {
+			BindNull(insertFileQuery_, file_table_column_names::target_file);
+		}
+
+		if (!extra_data->extraFlags_.empty()) {
+			Bind(insertFileQuery_, file_table_column_names::extra_flags, extra_data->extraFlags_);
+		}
+		else {
+			BindNull(insertFileQuery_, file_table_column_names::extra_flags);
+		}
+
+		if (!extra_data->persistentState_.empty()) {
+			Bind(insertFileQuery_, file_table_column_names::persistent_state, extra_data->persistentState_);
+		}
+		else {
+			BindNull(insertFileQuery_, file_table_column_names::persistent_state);
+		}
 	}
 	else {
 		BindNull(insertFileQuery_, file_table_column_names::target_file);
+		BindNull(insertFileQuery_, file_table_column_names::extra_flags);
 	}
 
 	int64_t localPathId = SaveLocalPath(file.GetLocalPath());
@@ -807,7 +868,6 @@ bool CQueueStorage::Impl::SaveFile(CFileItem const& file)
 	Bind(insertFileQuery_, file_table_column_names::local_path, localPathId);
 	Bind(insertFileQuery_, file_table_column_names::remote_path, remotePathId);
 
-	Bind(insertFileQuery_, file_table_column_names::download, file.Download() ? 1 : 0);
 	if (file.GetSize() != -1) {
 		Bind(insertFileQuery_, file_table_column_names::size, file.GetSize());
 	}
@@ -821,7 +881,7 @@ bool CQueueStorage::Impl::SaveFile(CFileItem const& file)
 		BindNull(insertFileQuery_, file_table_column_names::error_count);
 	}
 	Bind(insertFileQuery_, file_table_column_names::priority, static_cast<int>(file.GetPriority()));
-	Bind(insertFileQuery_, file_table_column_names::ascii_file, file.Ascii() ? 1 : 0);
+	Bind(insertFileQuery_, file_table_column_names::flags, static_cast<int64_t>(file.flags() - queue_flags::mask));
 
 	if (file.m_defaultFileExistsAction != CFileExistsNotification::unknown) {
 		Bind(insertFileQuery_, file_table_column_names::default_exists_action, file.m_defaultFileExistsAction);
@@ -860,7 +920,6 @@ bool CQueueStorage::Impl::SaveDirectory(CFolderItem const& directory)
 	Bind(insertFileQuery_, file_table_column_names::local_path, localPathId);
 	Bind(insertFileQuery_, file_table_column_names::remote_path, remotePathId);
 
-	Bind(insertFileQuery_, file_table_column_names::download, directory.Download() ? 1 : 0);
 	BindNull(insertFileQuery_, file_table_column_names::size);
 	if (directory.m_errorCount) {
 		Bind(insertFileQuery_, file_table_column_names::error_count, directory.m_errorCount);
@@ -869,7 +928,7 @@ bool CQueueStorage::Impl::SaveDirectory(CFolderItem const& directory)
 		BindNull(insertFileQuery_, file_table_column_names::error_count);
 	}
 	Bind(insertFileQuery_, file_table_column_names::priority, static_cast<int>(directory.GetPriority()));
-	BindNull(insertFileQuery_, file_table_column_names::ascii_file);
+	Bind(insertFileQuery_, file_table_column_names::flags, static_cast<int>(directory.flags() - queue_flags::mask));
 
 	BindNull(insertFileQuery_, file_table_column_names::default_exists_action);
 
@@ -917,6 +976,19 @@ std::string CQueueStorage::Impl::GetColumnTextUtf8(sqlite3_stmt* statement, int
 	return std::string();
 }
 
+std::string CQueueStorage::Impl::GetColumnBlob(sqlite3_stmt* statement, int index)
+{
+	int len = sqlite3_column_bytes(statement, index);
+	if (len) {
+		char const* text = reinterpret_cast<char const*>(sqlite3_column_blob(statement, index));
+		if (text) {
+			return std::string(text, len);
+		}
+	}
+
+	return std::string();
+}
+
 int64_t CQueueStorage::Impl::GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def)
 {
 	if (sqlite3_column_type(statement, index) == SQLITE_NULL) {
@@ -1050,7 +1122,7 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(Site & site)
 
 
 	site.server.SetBypassProxy(GetColumnInt(selectServersQuery_, server_table_column_names::bypass_proxy) == 1 );
-	site.server.SetName(GetColumnText(selectServersQuery_, server_table_column_names::name));
+	site.SetName(GetColumnText(selectServersQuery_, server_table_column_names::name).substr(0, 255));
 
 	fz::query_string qs = fz::query_string(GetColumnTextUtf8(selectServersQuery_, server_table_column_names::parameters));
 	for (auto const& pair : qs.pairs()) {
@@ -1077,7 +1149,8 @@ int64_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 	CLocalPath const localPath(GetLocalPath(localPathId));
 	CServerPath const remotePath(GetRemotePath(remotePathId));
 
-	bool download = GetColumnInt(selectFilesQuery_, file_table_column_names::download) != 0;
+	auto flags = static_cast<transfer_flags>(GetColumnInt(selectFilesQuery_, file_table_column_names::flags));
+	bool const download = flags & transfer_flags::download;
 
 	if (localPathId == -1 || remotePathId == -1) {
 		// QueueItemType::Folder
@@ -1099,7 +1172,9 @@ int64_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 		unsigned char errorCount = static_cast<unsigned char>(GetColumnInt(selectFilesQuery_, file_table_column_names::error_count));
 		int priority = GetColumnInt(selectFilesQuery_, file_table_column_names::priority, static_cast<int>(QueuePriority::normal));
 
-		bool ascii = GetColumnInt(selectFilesQuery_, file_table_column_names::ascii_file) != 0;
+		std::wstring extraFlags = GetColumnText(selectFilesQuery_, file_table_column_names::extra_flags);
+		std::string persistentState = GetColumnTextUtf8(selectFilesQuery_, file_table_column_names::persistent_state);
+
 		int overwrite_action = GetColumnInt(selectFilesQuery_, file_table_column_names::default_exists_action, CFileExistsNotification::unknown);
 
 		if (sourceFile.empty() || localPath.empty() ||
@@ -1110,9 +1185,8 @@ int64_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 			return INVALID_DATA;
 		}
 
-		CFileItem* fileItem = new CFileItem(0, true, download, sourceFile, targetFile, localPath, remotePath, size);
+		CFileItem* fileItem = new CFileItem(0, flags | queue_flags::queued, std::move(sourceFile), std::move(targetFile), std::move(localPath), std::move(remotePath), size, std::move(extraFlags), std::move(persistentState));
 		*pItem = fileItem;
-		fileItem->SetAscii(ascii);
 		fileItem->SetPriorityRaw(QueuePriority(priority));
 		fileItem->m_errorCount = errorCount;
 
@@ -1321,7 +1395,7 @@ bool CQueueStorage::Clear()
 
 std::wstring CQueueStorage::GetDatabaseFilename()
 {
-	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + L"queue.sqlite3";
+	return COptions::Get()->get_string(OPTION_DEFAULT_SETTINGSDIR) + L"queue.sqlite3";
 }
 
 bool CQueueStorage::BeginTransaction()
